@@ -5,6 +5,7 @@ import time
 from json import JSONDecodeError
 from typing import Optional, Union, Any, Type, List, Dict, Literal
 from urllib.parse import urlencode
+import warnings
 from dateutil import parser
 
 import requests
@@ -16,7 +17,7 @@ from kameleoon.defaults import DEFAULT_BLOCKING, DEFAULT_CONFIGURATION_PATH, DEF
 from kameleoon.version import __version__ as kameleoon_python_version
 
 from kameleoon.exceptions import ConfigurationNotFoundException, CredentialsNotFoundException, \
-    ExperimentConfigurationNotFound, NotTargeted, NotActivated, VariationConfigurationNotFound,\
+    ExperimentConfigurationNotFound, NotTargeted, NotActivated, SiteCodeDisabled, VariationConfigurationNotFound,\
     FeatureConfigurationNotFound, FeatureVariableNotFound
 from kameleoon.helpers.functions import check_visitor_code, obtain_hash_double, get_size, read_kameleoon_cookie_value
 from kameleoon.helpers.logger import get_logger
@@ -95,6 +96,10 @@ class KameleoonClient:
             self.tracking_base_url = "https://api-ssx.kameleoon.net/"
         else:
             self.tracking_base_url = "https://api-ssx.kameleoon.com/"
+        try:
+            self.environment = self.config['environment']
+        except KeyError:
+            self.environment = 'production'
         auth_resp = self._obtain_access_token()
         if auth_resp:
             self.access_token = auth_resp['access_token']
@@ -291,6 +296,10 @@ class KameleoonClient:
                           the user has been associated with excluded traffic
             NotTargeted: The visitor is not targeted by the experiment, as the associated targeting segment conditions
                          were not fulfilled. He should see the reference variation
+            VisitorCodeNotValid: Raise when the provided visitor code is not valid
+                        (empty, or longer than 255 characters)
+            SiteCodeDisabled: Raise when the siteCode is not disabled, SDK doesn't work with disabled siteCodes.
+                        To make SDK working please enable Site in your account.
 
 
         Examples:
@@ -359,6 +368,7 @@ class KameleoonClient:
                 variation = variation_id
             return variation
         else:
+            self._check_site_code_enable(experiment)
             visitor_data = self.data[visitor_code] if visitor_code in self.data else []
             if SEGMENT not in experiment or experiment[
                     SEGMENT] is None or experiment[SEGMENT].check_tree(visitor_data):
@@ -578,6 +588,10 @@ class KameleoonClient:
             NotTargeted: Exception indicating that the current visitor / user did not trigger
                          the required targeting conditions for this feature. The targeting conditions are defined
                          via Kameleoon's segment builder.
+            VisitorCodeNotValid: Raise when the provided visitor code is not valid
+                        (empty, or longer than 255 characters)
+            SiteCodeDisabled: Raise when the siteCode is not disabled, SDK doesn't work with disabled siteCodes.
+                        To make SDK working please enable Site in your account.
 
         Examples:
 
@@ -602,6 +616,7 @@ class KameleoonClient:
         """
         # pylint: disable=no-else-return
         check_visitor_code(visitor_code)
+        self._check_feature_key(feature_key)
         feature_flag = self.get_feature_flag(feature_key)
         data_not_sent = self._data_not_sent(visitor_code)
         if self.blocking:
@@ -632,11 +647,12 @@ class KameleoonClient:
                 visitor_data = self.data[visitor_code]
             except KeyError:
                 visitor_data = []
+            self._check_site_code_enable(feature_flag)
             if SEGMENT not in feature_flag or feature_flag[SEGMENT] is None or feature_flag[
                     SEGMENT].check_tree(visitor_data):
                 if self._feature_flag_scheduled(feature_flag, time.time()):
-                    threshold = obtain_hash_double(visitor_code, {}, id)
-                    if threshold <= feature_flag['expositionRate']:
+                    threshold = obtain_hash_double(visitor_code, {}, feature_flag['id'])
+                    if threshold >= 1 - feature_flag['expositionRate']:
                         variations_id = None
                         if feature_flag["variations"]:
                             variations_id = feature_flag["variations"][0]
@@ -707,6 +723,7 @@ class KameleoonClient:
         """
 
         # pylint: disable=no-else-raise
+        self._check_feature_key(feature_key)
         feature_flag = self.get_feature_flag(feature_key)
         custom_json = None
         try:
@@ -764,7 +781,7 @@ class KameleoonClient:
             self.access_token = auth_resp['access_token']
         self.logger.debug('Scheduled job to fetch configuration graphQL is starting.')
         self.obtain_experiments_graphql(self.site_code)
-        self.obtain_feature_flags_graphql(self.site_code)
+        self.obtain_feature_flags_graphql(self.site_code, self.environment)
 
     def _fetch_configuration_job(self) -> None:
         """
@@ -930,7 +947,11 @@ class KameleoonClient:
             self.logger.debug("Experiment are fetched: %s", self.experiments)
         return experiments
 
-    def obtain_feature_flags_graphql(self, site_code: str, per_page=-1) -> Optional[List[Dict[str, Any]]]:
+    def obtain_feature_flags_graphql(
+        self, site_code: str,
+        environment: str = 'production',
+        per_page=-1
+    ) -> Optional[List[Dict[str, Any]]]:
         """
         Obtain feature flags with GraphQL
         :param site_id:
@@ -939,7 +960,7 @@ class KameleoonClient:
         """
         self.logger.debug("Fetching feature flags")
         feature_flags = self._fetch_all_graphql(f"{self.automation_base_url}v1/graphql?perPage={per_page}",
-                                                data=query_ql_feature_flags(site_code),
+                                                data=query_ql_feature_flags(site_code, environment),
                                                 headers=self._get_headers())
         feature_flags = [feature_flag['node'] for feature_flag_QL in feature_flags
                          for feature_flag in feature_flag_QL['data']['featureFlags']['edges']]
@@ -1120,3 +1141,24 @@ class KameleoonClient:
                 if data_list:
                     data[vis_code] = [data_instance for data_instance in data_list if not data_instance.sent]
         return data
+
+    def _check_site_code_enable(self, exp_or_ff: Dict[str, Any]):
+        """
+        raise SiteCodeDisabled if site of Experiment or Feature Flag is disabled
+        :param exp_or_ff:
+        :type exp_or_ff: Dict[str, Any]
+        """
+        if exp_or_ff.get('site') is not None and not exp_or_ff['site']['isKameleoonEnabled']:
+            raise SiteCodeDisabled(self.site_code)
+
+    def _check_feature_key(self, feature_key: Union[str, int]):
+        """
+        warning user that feature_id is deprecated and need to pass feature_key with `str` type
+        :param feature_key:
+        :type feature_key: Union[str, int]
+        """
+        if isinstance(feature_key, int):
+            warnings.warn(
+                'Passing `feature_key` with type of `int` to `activate_feature` or `obtain_feature_variable` '
+                'is deprecated, it will be removed in next releases. This is necessary to support multi-environment '
+                'feature')
